#!/bin/ksh
#=======================================================================#
# Licensed Materials - Property of IBM                                  #
# 5724-C04                                                              #
# Copyright IBM Corporation 2006-2014 All Rights Reserved               #
#=======================================================================#
# db2connect.sh - This script connects to the TEPS data base (in DB2)   #
#    to verify that the password is correct.                            #
#-----------------------------------------------------------------------#
# SYNTAX: db2connect.sh <db2inst> <tepsdb> <tepsuser> <tepspassfile>    #
#                                                                       #
#    Where:                                                             #
#       <db2inst> is the DB2 instance name. Example: "db2inst1".        #
#       <tepsdb> is the name of the TEPS data base. Example: "TEPS".    #
#       <tepsuser> is the name of the login userid for the TEPS data    #
#          base. Example: "itmuser".                                    #
#       <tepspassfile> is the name of a file that contains the password #
#          for <tepsuser>. The file contains a single record in the     #
#          form "DB2PW=password".                                       #
#                                                                       #
# RETURN CODES:                                                         #
#    0 - The user has the correct password and was able to connect.     #
#    1 - The user has the incorrect password and was unable to connect. #
#                                                                       #
# NOTE: This script employs its own trace mechanism that is enabled by  #
#    the environment variable setting TRACE_LEVEL=DEBUG_MAX. If this    #
#    setting is in effect, a file "/tmp/db2connect-<PID>.log" is left   #
#    with details as to what happened for that particular run. We can't #
#    use the standard ITMSH_TRACE/itmsh_trace_main because the Java     #
#    caller eats the trace output.                                      #
#=======================================================================#

#-----------------------------------------------------------------------#
# Initialization.                                                       #
#-----------------------------------------------------------------------#
unalias -a;                                 # avert alias interference  #

db2connect_rc=0;                            # 0=access ok, 1=bad        #
db2connect_log="/tmp/db2connect-${$}.log";  # log/trace output          #
db2connect_tmp="/tmp/db2connect-${$}.tmp";  # private temporary file    #
unset DB2OPTIONS;                           # prevent funny stuff       #

print "db2connect args: ${@}" > "${db2connect_log}";

#-----------------------------------------------------------------------#
# Load the db2 environment for the instance specified.                  #
#-----------------------------------------------------------------------#
db2home=$(eval print "~$1")
ksh -c ". \"${db2home}/sqllib/profile.env\"" 2>> "${db2connect_log}";
if (( ${?} == 0 )); then
   . "${db2home}/sqllib/profile.env";
   ksh -c ". \"${db2home}/sqllib/db2profile\"" 2>> "${db2connect_log}";
   if (( ${?} == 0 )); then
      . "${db2home}/sqllib/db2profile";
   else
      db2connect_rc=1;
   fi;
else
   db2connect_rc=1;
fi;

#-----------------------------------------------------------------------#
# Connect to the TEPS data base. Note that we pass the CONNECT command  #
# via a temporary file that only we can read/write.                     #
#-----------------------------------------------------------------------#
if (( ${db2connect_rc} == 0 )); then
   DB2PW=$(grep "^DB2PW=" "${4}" 2>> "${db2connect_log}" | cut -d"=" -f2);
   
   if [[ "${DB2PW}" = "" ]]; then
      print "Error: Password not found in ${4}." >> "${db2connect_log}";
      db2connect_rc=1;                       # no password == no connect #
   else
      touch "${db2connect_tmp}";             # make the temp file        #
      chmod 600 "${db2connect_tmp}";         # make it private to us     #
      db2Connect="CONNECT TO ${2} USER ${3} USING";
      print "${db2Connect} <password_hidden>" >> "${db2connect_log}"; 
      print "${db2Connect} ${DB2PW}"          >> "${db2connect_tmp}"; 
      
      db2 -f "${db2connect_tmp}" >> "${db2connect_log}" 2>&1;
      db2RC=${?};
      rm -rf "${db2connect_tmp}" > /dev/null 2>&1;
      print "db2 connect rc=${db2RC}" >> "${db2connect_log}";
      
      if (( ${db2RC} != 0 )); then
         db2connect_rc=1;                   # connect failed            #
      else
         db2connect_rc=0;                   # connect succeeded         #
      fi;                                   # (if(db2RC!=0))            #
   fi;                                      # (else(DB2PW=""))          #
fi;                                         # (if(db2connect_rc==0))    #

#-----------------------------------------------------------------------#
# Finalize the log and return results to caller.                        #
#-----------------------------------------------------------------------#
if [[ "${TRACE_LEVEL}" != "DEBUG_MAX" ]]; then   # are we tracing?      #
   rm -rf "${db2connect_log}" > /dev/null 2>&1;  # nope, nuke the log   #
else
   print "db2connect rc: ${db2connect_rc}" >> "${db2connect_log}";
fi;

exit ${db2connect_rc};
