#!/bin/sh

#****************************************************************//
#* COPYRIGHT_COMMENT_TAG_BEGIN
#* Licensed Materials - Property of IBM (C)
#* Copyright IBM Corp. 2013, 2016. All Rights Reserved.                 #216691
#* US Government Users Restricted Rights - Use, duplication,
#* or disclosure restricted by GSA ADP Schedule Contract with
#* IBM Corp
#* COPYRIGHT_COMMENT_TAG_END
#****************************************************************//

myname=`basename $0`
curdate=`date +"%Y%m%d_%H%M"`
verbose=0

runcmd() {
    if [ $verbose -eq 1 ]; then
        eval $*
        retval=$?
    else
        eval $* > /dev/null 2>&1
        retval=$?
    fi

    return $retval
}

usage() {
    echo "Usage: $myName [-v] [-o <path_to_output_directory>] -i <path_to_installation_media_directory> -d <path_to_candle_home> -p <product_code1> [-p <product_code2>]"
    echo "       $myName -h"
}

cleanup() {
    runcmd rm -f $prstmplog
}

if [ $# -eq 0 ]; then
    usage
    exit 1
fi

# Solaris has different path for POSIX-compliant grep
GREPCMD=grep
if [ `uname -s` = "SunOS" ]; then
    GREPCMD="/usr/xpg4/bin/grep"
fi

# use nawk if available                                                 #216691
AWKCMD="awk";                                                           #216691
type nawk >/dev/null 2>&1;                                              #216691
[ $? -eq 0 ] && AWKCMD="nawk";                                        #216691
                                                                        #216691
while getopts "vo:i:d:p:h" opt
do
    case $opt in
        v)  verbose=1
            ;;
        o)  userprsdir=$OPTARG
            ;;
        i)  installdvd=$OPTARG
            ;;
        d)  candlehome=$OPTARG
            ;;
        p)  # Three types of product codes are accepted: 2-letter, 2-letter
            # concatenated with 32 to indicate 32-bit version and 2-letter with
            # 3264 concatenated to indicate that both 32 and 64-bit versions
            # should be checked for prerequisites
            if [ `echo $OPTARG | wc -c` -eq 3 ]; then
                productcodes="$productcodes $OPTARG"
            elif [ `echo $OPTARG | $GREPCMD -E '^[a-zA-Z0-9]{2}32$' >/dev/null 2>&1; echo $?` -eq 0 ]; then
                pc=`echo $OPTARG | cut -c 1,2 | tr '[a-z]' '[A-Z]' 2>/dev/null`
                productcodes="$productcodes $pc"
                eval K${pc}_32=true
                export K${pc}_32
            elif [ `echo $OPTARG | $GREPCMD -E '^[a-zA-Z0-9]{2}3264$' >/dev/null 2>&1; echo $?` -eq 0 ]; then
                pc=`echo $OPTARG | cut -c 1,2 | tr '[a-z]' '[A-Z]' 2>/dev/null`
                productcodes="$productcodes $pc"
                eval K${pc}_3264=true
                export K${pc}_3264
            fi
            ;;
        h)  usage
            exit 0
            ;;
        *)  usage
            exit 1
            ;;
    esac
done

# Validate input arguments

# Invalid installation media directory path disables prereqchecker usage.
if [ ! -d "$installdvd" ]; then
    runcmd usage
    exit 1
fi

if [ ! -d "$candlehome" ]; then
    runcmd usage
    exit 1
fi

# This is the default location for storing Prerequisite Scanner's output.
prsdir=${candlehome}/tmp/ITM_PRS_${curdate}

if [ -n "$userprsdir" ]; then
    # Search for relative paths (those with ./ at the beginning). They need
    # special care as prsdir is directly supplied to prereq_checker.sh and it
    # will create output directory in different current working dir than
    # prereqlauncher.sh thus confusing user as to the location of the output dir.
    echo $userprsdir | grep "^\.\/" > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        userprsdir=`pwd -P`/${userprsdir}
    fi
        prsdir=$userprsdir
fi

mkdir -p ${prsdir}
if [ ! -d ${prsdir} ]; then
    echo "Unable to create directory $prsdir. Check permissions." 1>&2 # $NON-NLS-1$
    exit 1
fi

prereqchecker=${installdvd}/unix/prereqchecker/prereq_checker.sh
prstmplog=`echo "${prsdir}/.prstmp.log" | tr -s "/"`
prsresult=`echo "${prsdir}/prs_result.txt" | tr -s "/"`
productcodes=`echo $productcodes | tr '[a-z]' '[A-Z]' 2>/dev/null`

# Call prereq_checker.sh for all components at once. This allows cumulative
# checking for disk space and memory
pcs=""
comma="";                                                               #216691
pcoutputdir=`echo "${prsdir}/COMMON/" | tr -s "/"`;                     #216691
pcoutputfile="${pcoutputdir}/result.txt";                               #216691
for pc in ${productcodes}; do                                           #216691
    # Check if PRS configuration file exists for a given component. If it doesn't
    # then exclude that component from the check.
    if [ -f ${installdvd}/unix/prereqchecker/K${pc}_codename.cfg ]; then
        pcs="${pcs}${comma}K${pc}"
        comma=",";                                                      #216691
    else
        echo "$pc;N/A;${pcoutputdir}" | tee -a "${prsresult}";          #216691
    fi
done

# CMVC defect 206742: prereq checker will return an error when run with empty pcs
if [ -n "$pcs" ]; then
    $prereqchecker "${pcs}" PATH=$candlehome outputdir="${pcoutputdir}" > ${prstmplog} 2>&1
    prsres=$?

    if [ ! -f "${pcoutputfile}" ]; then                                 #216691
        echo "ERR;ERROR;${pcoutputdir}"
    else                                                                #216691
        resultrec=`tr '[:lower:]' '[:upper:]' < "${pcoutputfile}" | ${GREPCMD} "^ *OVERALL RESULT:"`; #216691
        if [ -z "${resultrec}" ]; then                                  #216691
            echo "ERR;ERROR;${pcoutputdir}";                            #216691
        else                                                            #216691
            #-----------------------------------------------------------#216691
            # Read and parse the prereq checker output. It emits a      #216691
            # rather long listing, but we only care above one           #216691
            # line that looks like this:                                #216691
            #                                                           #216691
            #    Overall result: PASS (KAX vrmf: PASS, KLZ vrmf: PASS)  #216691
            #                                                           #216691
            # We translate it to output that looks like this:           #216691
            #                                                           #216691
            #    AX;PASS;<pcoutputdir>                                  #216691
            #    LZ;PASS;<pcoutputdir>                                  #216691
            #    pc:OVERALLPASS;<pcoutputdir>                           #216691
            #                                                           #216691
            # Up above, we use tr+grep to find and translate the one    #216691
            # record we need. Here we use sed+awk script to perform     #216691
            # perform the conversion:                                   #216691
            #                                                           #216691
            # - The sed command translates " (", ")", and ", " combos   #216691
            #   to "(". This is a launching ramp for awk to later       #216691
            #   translate all "(" to newlines.                          #216691
            #                                                           #216691
            #      OVERALL RESULT: PASS(KAX vrmf: PASS(KLZ vrmf: PASS(  #216691
            #                                                           #216691
            #   Note: Some platforms that ITM supports provide an awk   #216691
            #      that only allows for a single character for the      #216691
            #      record separator (RS). Otherwise, we could have      #216691
            #      obviated the sed call by specifying the awk record   #216691
            #      separator in the form of a pattern.                  #216691
            #                                                           #216691
            # - The awk script processes the sed output as follows:     #216691
            #                                                           #216691
            #    BEGIN - setup before any input is read:                #216691
            #      - the RS assignment splits each record into records  #216691
            #        based on "("                                       #216691
            #      - the FS assignment splits each record into fields   #216691
            #        based on ":" plus one or more spaces               #216691
            #      - resulting breakdown:                               #216691
            #                  field 1:            field 2:             #216691
            #        record 1: "OVERALL RESULT"   "PASS"                #216691
            #        record 2: "KAX xxx"          "PASS"                #216691
            #        record 3: "KLZ xxx"          "PASS"                #216691
            #                                                           #216691
            #    /^K/ - for each record starting with "K":              #216691
            #      - print "xx:yyyy;pcoutputdir"                        #216691
            #      - where "xx" is from "Kxx nnnnnnnn" and "yyyy" is    #216691
            #        PASS/FAIL                                          #216691
            #                                                           #216691
            #    /^OVERALL/ - for each record starting with "OVERALL"   #216691
            #        (there's only one):                                #216691
            #      - save PASS/FAIL value for later                     #216691
            #                                                           #216691
            #    END - after all records processed                      #216691
            #      - print "pc:OVERALLyyyy;pcoutputdir"                 #216691
            #      - where "yyyy" is PASS/FAIL from "Overall" record    #216691
            #        above                                              #216691
            #-----------------------------------------------------------#216691
                        awkscr="BEGIN     { RS=\"(\"; FS=\": +\" }";                               #216691
            awkscr="${awkscr}; /^K/       { print substr(\$1,2,2)\";\"\$2\";\"pcoutputdir }";      #216691
            awkscr="${awkscr}; /^OVERALL/ { overall_status=\$2 }";                                 #216691
            awkscr="${awkscr}; END        { print \"pc;OVERALL\"overall_status\";\"pcoutputdir }"; #216691
            echo "${resultrec}" | sed 's/ *[(),] */(/g' | ${AWKCMD} -v "pcoutputdir=${pcoutputdir}" "${awkscr}" | tee -a "${prsresult}"; #216691
        fi;                                                             #216691
    fi;                                                                 #216691
else
    # In case there are no PRS config files then fake overall PASS result so
    # CandleInstall.java can go further.
    echo "pc;OVERALLPASS;${pcoutputdir}" | tee -a "${prsresult}";       #216691
fi

cleanup
exit 0