#!/bin/ksh
#=======================================================================
# wasihs855uplift.sh - This script installs ITM's WAS v855 within a
#   given CANDLEHOME and migrates the eWas v80X profile to Was v855.
#-----------------------------------------------------------------------
# WARNING:
#       The user must backup CANDLEHOME before running this program
#       to perform the update.
#-----------------------------------------------------------------------
# SYNTAX: wasihs855uplift.sh [-h CANDLEHOME] [-i IMAGE_DIR] [-t no|yes]
#
#    Where:
#       -h: Specifies the ITM installation directory. If not
#           specified, wasihs855uplift assumes /opt/IBM/ITM.
#       -i: Specifies the directory where the ITM WAS v855 and IHS v855
#           images reside (wasihs855uplift_<platform>_v.rr.mm.ff).
#           If not specified, wasihs855uplift assumes the image resides
#           in the same directory where wasihs855uplift itself resides.
#       -t: Indicates test mode. In test mode, wasihs855uplift looks
#           for and reports on the environment to be updated, but does
#           not perform the update. If not specified, wasihs855uplift
#           does not run in test mode.
#
# RETURN CODES:
#    0 - Success.
#    2 - Syntax error.
#    3 - No WAS found in image directory or CANDLEHOME.
#    4 - Other init problem: IIM doesn't run, TEPS is running, etc.
#    5 - Error occurred while updating package in CANDLEHOME.
#    6 - Error occurred while updating the *.ver file.
#=======================================================================

function usage {

   #--------------------------------------------------------------------
   # Display syntax.
   #--------------------------------------------------------------------
   print -u2 "Usage: ${pgm} [-h CANDLEHOME] [-i IMAGE_DIR] [-t no|yes]";
}                                           # (usage())

function resolve_dir {

#-----------------------------------------------------------------------
# Perform directory canonicalization. In other words, the input
# directory string is converted to a fully qualified name (no
# wildcards, "..", ".", and so on. For example, "/tmp/.." resolves
# to "/". The input directory string must refer to a directory that
# exists.
#-----------------------------------------------------------------------
   typeset result;                          # localize output variable
   result=$(readlink -f ${1} 2> /dev/null); # try readlink
   if [[ ${?} -ne 0 ]]; then                # if no readlink
      cd -P ${1} > /dev/null 2>&1;          # use cd to change/resolve
      if [[ "${myos}" = "SunOS" ]]; then    # $PWD/pwd shell broken
         result=$(/usr/bin/pwd);            # use pwd binary
      else                                  # $PWD is ok
         result="${PWD}";                   # use its value
      fi;                                   # (elif($myos="SunOs"))
      cd ${OLDPWD} > /dev/null 2>&1;        # restore working directory
   fi;                                      # (if($?!=0))
   print "${result}";                       # pass result to caller
}                                           # (resolve_dir())

function parse_parameters {
   typeset myrc=0;                          # assume success
   #--------------------------------------------------------------------
   # Parse parameter list.
   # Gather the options and verifying they are syntactically correct.
   #--------------------------------------------------------------------
   [[ -z "${CANDLEHOME}" ]] && CANDLEHOME="/opt/IBM/ITM"; # location to update
   image_dir=${pgm_dir};                    # source of images
   typeset -l test_opt="no";                # assume live mode
   while getopts ":h:i:t:" opt > /dev/null 2>&1 && (( ${myrc} == 0 )); do
      case "${opt}" in
         "h") CANDLEHOME="${OPTARG}";;
         "i") image_dir="${OPTARG}";;
         "t") test_opt="${OPTARG}";;
         ":") print -u2 "${pgm}: -${OPTARG} value is missing.";
              myrc=2;;
         "?") if [[ "${OPTARG}" = "?" ]]; then
                 usage;
              else
                 print -u2 "${pgm}: Option -${OPTARG} unknown.";
              fi;
              myrc=2;;
      esac;                                 # (case(opt...))
   done;                                    # (while(getopts...))
   test_mode=${test_opt};
   if (( ${OPTIND} <= ${#} )) && (( ${myrc} == 0 )); then
      shift $(( ${OPTIND} - 1));
      print -u2 "${pgm}: Argument \"${1}\" unknown.";
      myrc=2;
   fi;                                      # (if($OPTIND<=$#))

   #--------------------------------------------------------------------
   # Check parameter values. Make sure what we asked for is valid.
   #--------------------------------------------------------------------
   if (( ${myrc} == 0 )); then
      if [[ ! -e "${CANDLEHOME}" ]]; then
         print -u2 "${pgm}: -h value '${CANDLEHOME}' does not exist.";
         myrc=2;
      elif [[ ! -d "${CANDLEHOME}" ]]; then
         print -u2 "${pgm}: -h value '${CANDLEHOME}' is not a directory.";
         myrc=2;
      elif [[ ! -w "${CANDLEHOME}" ]]; then
         print -u2 "${pgm}: -h value '${CANDLEHOME}' is not writable.";
         myrc=2;
      elif [[ ! -e "${image_dir}" ]]; then
         print -u2 "${pgm}: -i value '${image_dir}' does not exist.";
         myrc=2;
      elif [[ ! -d "${image_dir}" ]]; then
         print -u2 "${pgm}: -i value '${image_dir}' is not a directory.";
         myrc=2;
      elif [[ ! -r "${image_dir}" ]]; then
         print -u2 "${pgm}: -i value '${image_dir}' is not readable.";
         myrc=2;
      elif [[ "${test_mode}" != @(yes|no) ]]; then
         print -u2 "${pgm}: -t value must be 'yes' or 'no'.";
         myrc=2;
      fi;                                   # (elif($test_mode!=ye...))
   fi;                                      # (if($myrc==0))
   return ${myrc};                          # return code to caller
}

function verify_auth {
   typeset myrc=0;                          # assume success
   #--------------------------------------------------------------------
   # Make sure we have access rights to the existing installation.
   #--------------------------------------------------------------------
   cIIM_owner_group=$(ls -dl "${cIIM}" | tr -s ' ' | cut -f3-4 -d' ');
   cIIM_owner=$(echo ${cIIM_owner_group} | cut -f1 -d' ');
   cIIM_group=$(echo ${cIIM_owner_group} | cut -f2 -d' ');
   if [[ "${cIIM_owner}" != "${myeuid}" ]]; then
      print -u2 "${pgm}: ${cIIM} is owned by ${cIIM_owner}:${cIIM_group}.";
      print -u2 "${pgm}: You are running ${pgm} with userid ${myeuid}.";
      print -u2 "${pgm}: You must run ${pgm} with userid ${cIIM_owner}.";
      myrc=4;
   elif [[ "${myeuid}" != "root" ]] && [[ " ${mygrps} " != *" ${cIIM_group} "* ]]; then
      print -u2 "${pgm}: ${cIIM} is owned by ${cIIM_owner}:${cIIM_group}.";
      print -u2 "${pgm}: ${myeuid} is not a member of group ${cIIM_group}.";
      myrc=4;
   fi;
   cWAS_owner_group=$(ls -dl "${cWAS}" | tr -s ' ' | cut -f3-4 -d' ');
   cWAS_owner=$(echo ${cWAS_owner_group} | cut -f1 -d' ');
   cWAS_group=$(echo ${cWAS_owner_group} | cut -f2 -d' ');
   if [[ "${cWAS_owner}" != "${myeuid}" ]]; then
      print -u2 "${pgm}: ${cWAS} is owned by ${cWAS_owner}:${cWAS_group}.";
      print -u2 "${pgm}: You are running ${pgm} with userid ${myeuid}.";
      print -u2 "${pgm}: You must run ${pgm} with userid ${cWAS_owner}.";
      myrc=4;
   elif [[ "${myeuid}" != "root" ]] && [[ " ${mygrps} " != *" ${cWAS_group} "* ]]; then
      print -u2 "${pgm}: ${cWAS} is owned by ${cWAS_owner}:${cWAS_group}.";
      print -u2 "${pgm}: ${myeuid} is not a member of group ${cWAS_group}.";
      myrc=4;
   fi;
   cIHS_owner_group=$(ls -dl "${cIHS}" | tr -s ' ' | cut -f3-4 -d' ');
   cIHS_owner=$(echo ${cIHS_owner_group} | cut -f1 -d' ');
   cIHS_group=$(echo ${cIHS_owner_group} | cut -f2 -d' ');
   if [[ "${cIHS_owner}" != "${myeuid}" ]]; then
      print -u2 "${pgm}: ${cIHS} is owned by ${cIHS_owner}:${cIHS_group}.";
      print -u2 "${pgm}: You are running ${pgm} with userid ${myeuid}.";
      print -u2 "${pgm}: You must run ${pgm} with userid ${cWAS_owner}.";
      myrc=4;
   elif [[ "${myeuid}" != "root" ]] && [[ " ${mygrps} " != *" ${cIHS_group} "* ]]; then
      print -u2 "${pgm}: ${cIHS} is owned by ${cIHS_owner}:${cIHS_group}.";
      print -u2 "${pgm}: ${myeuid} is not a member of group ${cIHS_group}.";
      myrc=4;
   fi;

   # Use the jar command from the eWas/WAS JDK.
   tmpJarCMD=${cWAS}/java/bin/jar;
   if [[ -f "${tmpJarCMD}" ]]; then
      if [[ -x "${tmpJarCMD}" ]]; then
         jarCMD=${tmpJarCMD};
      else
         if [[ "${test_mode}" = "no" ]]; then
            print "${pgm}: Repairing jar permissions.";
            chmod a+x "${tmpJarCMD}";
            rc=${?};
            if (( ${rc} == 0 )); then
               print "${pgm}: Permissions repaired successfully.";
            else
               print -u2 "${pgm}: Permissions repair failed.";
               print -u2 "${pgm}: 'chmod a+x \"${tmpJarCMD}\"' failed (rc=${rc}).";
               myrc=4;
            fi;                             # (if($rc!=0))
         fi;                                # (else($test_mode!=no))
      fi
   else
      print -u2 "${pgm}: Command not found: ${tmpJarCMD}.";
      myrc=4;
   fi

   # Use the java command from the eWas/WAS JRE.
   tmpJavaCMD=${cWAS}/java/jre/bin/java;
   if [[ -f "${tmpJavaCMD}" ]]; then
      if [[ -x "${tmpJavaCMD}" ]]; then
         javaCMD=${tmpJavaCMD};
      else
         if [[ "${test_mode}" = "no" ]]; then
            print "${pgm}: Repairing java permissions.";
            chmod a+x "${tmpJarCMD}";
            rc=${?};
            if (( ${rc} == 0 )); then
               print "${pgm}: Permissions repaired successfully.";
            else
               print -u2 "${pgm}: Permissions repair failed.";
               print -u2 "${pgm}: 'chmod a+x \"${tmpJavaCMD}\"' failed (rc=${rc}).";
               myrc=4;
            fi;                             # (if($rc!=0))
         fi;                                # (else($test_mode!=no))
      fi
   else
      print -u2 "${pgm}: Command not found: ${tmpJavaCMD}.";
      myrc=4;
   fi
   return ${myrc};                          # return code to caller
}

function determine_updates {
   typeset myrc=0;                          # assume success
   #--------------------------------------------------------------------
   # Determine whether a component needs to be updated. When we checked
   # the media image, we got the new version and when we isolated the
   # currently installed component, we got the old version. So, now we
   # compare the two versions to make sure the update makes sense.
   #--------------------------------------------------------------------

   #--------------------------------------------------------------------
   # Determine whether WAS needs to be updated.
   #--------------------------------------------------------------------
   if [[ "${cWAS_plat}" = "li6263" ]]; then
      print "${pgm}: Skipping iw${cWAS_plat}: Upgrade of 32 bit platform ${cWAS_plat} is not supported.";
      print "${pmg}: Refer to https://www.ibm.com/support/knowledgecenter/SSTFXA_6.3.0/com.ibm.itm.doc_6.3/install/teps64.htm"
      update_iw=-1;                         # don't update and don't post-config
   elif [[ "${iWAS_VRMF}" > "${cWAS_VRMF}" ]]; then
      update_iw=1;                          # update of existing VRMF
   elif [[ "${iWAS_VRMF}" = "${cWAS_VRMF}" ]]; then
      print "${pgm}: Skipping iw${cWAS_plat}: image version ${iWAS_VRMF} is the same as the current version ${cWAS_VRMF}.";
   else
      print "${pgm}: Skipping iw${cWAS_plat}: image version ${iWAS_VRMF} is older than current version ${cWAS_VRMF}.";
   fi;                                      # (else(iWAS_VRMF...))
   #-----------------------------------------------------------------------
   # Determine whether IHS needs to be updated.
   #-----------------------------------------------------------------------
   if [[ "${cIHS_plat}" = "li6263" ]]; then
      print "${pgm}: Skipping iu${cIHS_plat}: Upgrade of 32 bit platform ${cIHS_plat} is not supported.";
      print "${pmg}: Refer to https://www.ibm.com/support/knowledgecenter/SSTFXA_6.3.0/com.ibm.itm.doc_6.3/install/teps64.htm"
      update_iu=-1;                          # don't update and don't post-config
   elif [[ "${iIHS_VRMF}" > "${cIHS_VRMF}" ]]; then
      update_iu=1;                          # upgrade of existing VRMF
   elif [[ "${iIHS_VRMF}" = "${cIHS_VRMF}" ]]; then
      print "${pgm}: Skipping iu${cIHS_plat}: image version ${iIHS_VRMF} is the same as the current version ${cIHS_VRMF}.";
   else
      print "${pgm}: Skipping iu${cIHS_plat}: image version ${iIHS_VRMF} is older than current version ${cIHS_VRMF}.";
   fi;                                      # (else(iIHS_VRMF...))
   return ${myrc};                          # return code to caller
}

function check_TEPS_running {
   typeset myrc=0;                          # assume success
   typeset cmd="";
   #---------------------------------------------------------------------
   # Make sure the TEPS isn't running.
   #---------------------------------------------------------------------
   x=$("${CANDLEHOME}/bin/cinfo" -r);
   rc=${?};
   if (( ${rc} != 0 )); then
      print -u2 "${pgm}: Unable to check portal server status.";
      print -u2 "${pgm}: 'cinfo -r' (rc=${rc}).";
      myrc=4;
   else
      cinfo_cq=$(print -n -R "${x}" | grep ".* *cq *.*\.\.\.running");
      cinfo_kf=$(print -n -R "${x}" | grep ".* *kf *.*\.\.\.running");
      if [[ -n "${cinfo_cq}" ]]; then
         print -u2 "${pgm}: The portal server is still running:\n${cinfo_cq}";
         print -u2 "${pgm}: Stop the portal server before using ${pgm} to update WAS and IHS.";
         myrc=4;
      fi;                                   # (if($cinfo_cq!=""))
      if [[ -n "${cinfo_kf}" ]]; then
         print -u2 "${pgm}: The help server is still running:\n${cinfo_kf}";
         print -u2 "${pgm}: Stop the help server before using ${pgm} to update WAS and IHS.";
         myrc=4;
      fi;                                   # (if($cinfo_kf!=""))
   fi;                                      # (else($rc!=0))
   if [[ "${test_mode}" != "no" ]]; then
      myrc=0;                            # this is ok in test mode
   fi;                                      # (if($test_mode!=no))
   return ${myrc};                          # return code to caller
}

function logging_start {
   typeset myrc=0;                          # assume success
   #--------------------------------------------------------------------
   # Start logging. From here on, all STDOUT and STDERR is written to
   # a log file so a) we leave a footprint and b) we have tracks for
   # problem diagnosis.
   #--------------------------------------------------------------------
   logdir="${CANDLEHOME}/logs";
   logdate=$(date +"%Y%m%d%H%M%S");         # yyyymmddhhmmss
   logfile="${logdir}/${pgm%.*}-${logdate}.log";
   exec 3>&1;                               # backup STDOUT
   exec 4>&2;                               # backup STDERR
   tee "${logfile}" >&3 |&                  # start logging
   logpid="${!}";                           # save tee's PID for later
   exec 1>&p;                               # redirect STDOUT to log
   exec 2>&1;                               # redirect STDERR to log
   #-----------------------------------------------------------------------
   # Report everything we know.
   #-----------------------------------------------------------------------
   print "";
   print "${pgm}: Parameters:";
   print "${pgm}: -h (CANDLEHOME):        ${CANDLEHOME}";
   print "${pgm}: -i (image directory):   ${image_dir}";
   print "${pgm}: -t (test mode):         ${test_mode}";
   print "${pgm}:";
   print "${pgm}: Platform:               ${myplat} (${myos}/${mymach})";
   print "${pgm}: Effective user+groups:  ${myeuid}:${mygrps}";
   print "${pgm}:";
   print "${pgm}: IIM version:            ${cIIM_VRMF}";
   print "${pgm}: IIM imcl command:       ${imcl_cmd}";
   print "${pgm}:";
   print "${pgm}: WAS Source directory:   ${iWAS}";
   print "${pgm}: WAS New version:        ${iWAS_VRMF}";
   print "${pgm}: WAS Old version:        ${cWAS_VRMF}";
   print "${pgm}: WAS Target directory:   ${cWAS}";
   print "${pgm}: WAS Target owner+group: ${cWAS_owner}:${cWAS_group}";
   print "${pgm}:";
   print "${pgm}: IHS Source directory:   ${iWAS}";
   print "${pgm}: IHS New version:        ${iIHS_VRMF}";
   print "${pgm}: IHS Old version:        ${cIHS_VRMF}";
   print "${pgm}: IHS Target directory:   ${cIHS}";
   print "${pgm}: IHS Target owner+group: ${cIHS_owner}:${cIHS_group}";
   print "${pgm}:";
   print "${pgm}: Log file:               ${logfile}";

   #-----------------------------------------------------------------
   # List Available Packages in the repos
   #-----------------------------------------------------------------
   # Need an empty temporary IMData location in case IM is already
   # installed for another product.
   TMPData=${CANDLEHOME}/tmp/IM/Data
   [ ! -d "${TMPData}" ] && mkdir -p ${TMPData}
   cmd="${imcl_cmd}";
   cmd="${cmd} listAvailablePackages";
   cmd="${cmd} -repositories '${iWAS_repositories}'";
   cmd="${cmd} -dataLocation '${TMPData}'";
   print "";
   WASnames="$(eval ${cmd})";
   rc=${?};
   if (( ${rc} != 0 )); then
      print "${pgm}: Command: ${cmd}";
      print -u2 "${pgm}: Error obtaining WAS package names from '${iWAS}' (rc=${rc}).";
      myrc=5;
   else
      if [[ -z "${WASnames}" ]]; then
         print "${pgm}: Command: ${cmd}";
         print -u2 "${pgm}: Error obtaining WAS package names from '${ieWAS}'.";
         myrc=5;
      else
         if [[ "${WASnames}" == @(*CRIMC1011W*) ]]; then
            print "${pgm}: Command: ${cmd}";
            print -u2 "${pgm}: Error obtaining WAS package names from '${iWAS}'.";
            print "${WASnames}" | while read line ; do
               print "${pgm}:                         $(print ${line} | sed 's/[^:]*: //')";
            done
            myrc=5;
         else
            print "${pgm}: Available package names:";
            print "${WASnames}" | while read line ; do
               print "${pgm}:                         $(print ${line} | sed 's/[^:]*: //')";
            done
         fi;
      fi;                                   # (else($pkgname=""))
   fi;                                      # (else($rc!=0))
   [ -d "${TMPData}" ] && rm -rf ${TMPData}

   return ${myrc};                          # return code to caller
}

function logging_stop {
   typeset myrc=0;                          # assume success
   #-----------------------------------------------------------------------
   # Stop writing log file and copy to *_failed dir if possible.
   #-----------------------------------------------------------------------
   if [[ "${logfile}" != "None" ]]; then
      exec 1>&3;                            # restore STDOUT
      exec 2>&4;                            # restore STDERR
      exec 3>&-;                            # close STDOUT backup
      exec 4>&-;                            # close STDERR backup

      wait "${logpid}";                     # wait for log to flush

      if ((${backup_iw} == 2)); then        # update failed
         [[ -d "${failed_iw_dir}" ]] || mkdir "${failed_iw_dir}"
         cp -p "${logfile}" "${failed_iw_dir}"; # copy for prob diagnosis
      fi;                                   # (if($backup_iw==2))
      if ((${backup_iu} == 2)); then        # update failed
         [[ -d "${failed_iu_dir}" ]] || mkdir "${failed_iu_dir}"
         cp -p "${logfile}" "${failed_iu_dir}"; # copy for prob diagnosis
      fi;                                   # (if($backup_iu==2))
   fi;                                      # (if($logfile!="None"))
   return ${myrc};                          # return code to caller
}

function find_imcl {
   typeset myrc=0;                          # assume success
   #--------------------------------------------------------------------
   # Find the imcl command and make sure it works ok by running "imcl
   # -version". We assume the version itself is ok we're just
   # checking whether imcl runs.
   #--------------------------------------------------------------------
   imcl_cmd="${cIIM}/tools/imcl";
   if [[ ! -a "${imcl_cmd}" ]]; then
      print -u2 "${pgm}: ${imcl_cmd} does not exist.";
      myrc=4;
   else
      if [[ ! -x "${imcl_cmd}" ]]; then
         print "${pgm}: Repairing imcl permissions.";
         chmod -R a+x "${CANDLEHOME}/${cIIM_plat}/ii";
         rc=${?};
         if (( ${rc} == 0 )); then
            print "${pgm}: Permissions repaired successfully.";
         else
            print -u2 "${pgm}: Permissions repair failed.";
            print -u2 "${pgm}: 'chmod -R a+x \"${CANDLEHOME}/${cIIM_plat}/ii\"' failed (rc=${rc}).";
            myrc=4;
         fi;                             # (if($rc!=0))
      fi;                                   # (if(!-x $imcl_cmd))

      if [[ -x "${imcl_cmd}" ]]; then
         ${imcl_cmd} -version > /dev/null 2>&1;
         rc=${?};
         if (( ${rc} != 0 )); then
            print -u2 "${pgm}: '${imcl_cmd} -version' failed (rc=${rc}).";
            myrc=4;
         fi;                                # (if($rc!=0))
      else
         print -u2 "${pgm}: ${imcl_cmd} is not executable.";
         myrc=4;
      fi;                                   # (else($imcl...))
   fi;                                      # (else($imcl_cmd))
   [[ "${test_mode}" != "no" ]] && myrc=0;  # this is ok for test
   return ${myrc};                          # return code to caller
}

function update_VerFile {
   typeset myrc=0;                          # assume success
   typeset verFile="$1";                    # Get ver file name
   typeset VRMF="$2";                       # Get new version
   #--------------------------------------------------------------------
   # Finally, we need to update the ITM registry to show the new
   # version and its install date.
   #--------------------------------------------------------------------
   if [[ "${test_mode}" = "no" ]]; then
      print "";
      print "${pgm}: Updating version information in ${verFile}.";
      oldver="${CANDLEHOME}/old_registry/"$(basename "${verFile}");
      cmd="cp -p \"${verFile}\" \"${oldver}\"";
      eval "${cmd}";
      rc=${?};
      if (( ${rc} != 0 )); then
         print -u2 "${pgm}: Error copying '${verFile}' to '${oldver}' (rc=${rc}).";
         myrc=5;
      else
         sedscr="'s/^VRMF.*=.*$/VRMF = "${VRMF}"/;/^installDate.*=.*$/ d'";
         eval sed "${sedscr}" < "${oldver}" > "${verFile}";
         rc=${?};
         if (( ${rc} != 0 )); then
            print -u2 "${pgm}: Sed error updating '${verFile}' (rc=${rc}).";
            myrc=5;
         else
            print "installDate = ${myeuid}|"$(date -u +"%s")"000|" >> "${verFile}"
            print "${pgm}: Updating ${verFile} complete.";
         fi;                                # (else($rc!=0))
      fi;                                   # (else($rc!=0))
   else
      print "${pgm}: Skip updating version information in ${verFile} in test mode.";
   fi;                                      # (else($test_mode==no))
   return ${myrc};                          # return code to caller
}

function reportVRMF {
   typeset myrc=0;                          # assume success
   typeset PP="$1";                         # Get product code
   print "${pgm}: \"cinfo -t ${PP}\" output:"
   eval "${CANDLEHOME}/bin/cinfo -t ${PP}";
   return ${myrc};                          # return code to caller
}

function parse_installation {
   typeset myrc=0;                          # assume success
   #--------------------------------------------------------------------
   # Find the currently installed versions of eWAS, IHS and IIM.
   # Out of this, we get these values:
   #
   #   cIIM      = ${CANDLEHOME}/<cIIM_plat>/ii
   #   cIIM_plat = ITM platform (aix536, li6263, lx8266, or ls3263)
   #               *This is part of the path where we find IIM.
   #   cIIM_ver  = ${CANDLEHOME}/registry/ii<cIIM_plat>.ver
   #               *We need to update the version here if the update
   #                succeeds.
   #   cIIM_VRMF = vvrrmmff from the ii<cIIM_plat>.ver file
   #               *We need this to compare with the image version to
   #                make sure the update is valid for this CANDLEHOME.
   #   cIIM_bit  = 32 or 64
   #
   #   cWAS      = ${CANDLEHOME}/<cWAS_plat>/iw
   #   cWAS_plat = ITM platform (aix536, li6263, lx8266, or ls3266)
   #               *This is part of the path where we find/install WAS.
   #   cWAS_ver  = ${CANDLEHOME}/registry/iw<cWAS_plat>.ver
   #               *We need to update the version here if the update
   #                succeeds.
   #   cWAS_VRMF = vvrrmmff from the iw<cWAS_plat>.ver file
   #               *We need this to compare with the image version to
   #                make sure the update is valid for this CANDLEHOME.
   #   cWAS_bit  = 32 or 64
   #
   #   cIHS      = ${CANDLEHOME}/<cIHS_plat>/iu
   #   cIHS_plat = ITM platform (aix536, li6263, lx8266, or ls3266)
   #               *This is part of the path where we find/install IHS.
   #   cIHS_ver  = ${CANDLEHOME}/registry/iu<cIHS_plat>.ver
   #               *We need to update the version here if the update
   #                succeeds.
   #   cIHS_VRMF = vvrrmmff from the iu<cIHS_plat>.ver file
   #               *We need this to compare with the image version to
   #                make sure the update is valid for this CANDLEHOME.
   #   cIHS_bit  = 32 or 64
   #--------------------------------------------------------------------

   case "${myplat}" in                      # *.ver search pattern
      AIX)    cWAS_ver="iwaix[0-9][0-9][0-9].ver"
              cIHS_ver="iuaix[0-9][0-9][0-9].ver"
              cIIM_ver="iiaix[0-9][0-9][0-9].ver"
      ;;
      XLINUX) cWAS_ver="iwl[xi][0-9][0-9][0-9][0-9].ver"
              cIHS_ver="iul[xi][0-9][0-9][0-9][0-9].ver"
              cIIM_ver="iil[xi][0-9][0-9][0-9][0-9].ver"
      ;;
      ZLINUX) cWAS_ver="iwls[0-9][0-9][0-9][0-9].ver"
              cIHS_ver="iuls[0-9][0-9][0-9][0-9].ver"
              cIIM_ver="iils[0-9][0-9][0-9][0-9].ver"
      ;;
   esac;                                    # (case(myplat))

   set -A cIIMs $(cd "${CANDLEHOME}/registry" ; ls -d ${cIIM_ver} 2> /dev/null);
   if (( "${#cIIMs[*]}" == 0 )); then
      print -u2 "${pgm}: No IIM packages found in CANDLEHOME for platform '${myplat}'.";
      myrc=3;
   elif (( "${#cIIMs[*]}" > 1 )); then
      print -u2 "${pgm}: Too many IIM packages found in CANDLEHOME for platform '${myplat}'.";
      myrc=3;
   else
      cIIM_ver="${CANDLEHOME}/registry/${cIIMs[0]}";
      cIIM_plat="${cIIMs[0]%.ver}";
      cIIM_plat="${cIIM_plat#ii}";
      if [[ "${cIIM_plat}" = *6 ]]; then
         cIIM_bit="64";
      else
         cIIM_bit="32";
      fi;
      cIIM="${CANDLEHOME}/${cIIM_plat}/ii";
      cIIM_VRMF=$(grep "^VRMF *=" "${cIIM_ver}" | cut -f2 -d'=' | tr -d ' ');
      if [[ "${cIIM_VRMF}" < "01090100" ]]; then
         print -u2 "${pgm}: Current IIM version ${cIIM_VRMF} is too old. Apply ITM service pack 6.30.07.00 SP5 or later first.";
         myrc=3;
      elif [[ ! -r "${cIIM}" ]]; then
         print -u2 "${pgm}: Unable to access ${cIIM}.";
         myrc=3;
      fi;
   fi;                                      # (else($cIIMs...))

   set -A cWASs $(cd "${CANDLEHOME}/registry" ; ls -d ${cWAS_ver} 2> /dev/null);
   if (( "${#cWASs[*]}" == 0 )); then
      print -u2 "${pgm}: No WAS packages found in CANDLEHOME for platform '${myplat}'.";
      myrc=3;
   elif (( "${#cWASs[*]}" > 1 )); then
      print -u2 "${pgm}: Too many WAS packages found in CANDLEHOME for platform '${myplat}'.";
      myrc=3;
   else
      cWAS_ver="${CANDLEHOME}/registry/${cWASs[0]}";
      cWAS_plat="${cWASs[0]%.ver}";
      cWAS_plat="${cWAS_plat#iw}";
      if [[ "${cWAS_plat}" = *6 ]]; then
         cWAS_bit="64";
      else
         cWAS_bit="32";
      fi;
      cWAS="${CANDLEHOME}/${cWAS_plat}/iw";
      cWAS_VRMF=$(grep "^VRMF *=" "${cWAS_ver}" | cut -f2 -d'=' | tr -d ' ');
      if [[ "${cWAS_VRMF}" < "08000600" ]]; then
         print -u2 "${pgm}: Current WAS version ${cWAS_VRMF} is too old. Apply ITM fix pack 6.30.02.00 or later first.";
         myrc=3;
      elif [[ ! -r "${cWAS}" ]]; then
         print -u2 "${pgm}: Unable to access ${cWAS}.";
         myrc=3;
      fi;
   fi;                                      # (else($cWASs...))

   set -A cIHSs $(cd "${CANDLEHOME}/registry" ; ls -d ${cIHS_ver} 2> /dev/null);
   if (( "${#cIHSs[*]}" == 0 )); then
      print -u2 "${pgm}: No IHS packages found in CANDLEHOME for platform '${myplat}'.";
      myrc=3;
   elif (( "${#cIHSs[*]}" > 1 )); then
      print -u2 "${pgm}: Too many IHS packages found in CANDLEHOME for platform '${myplat}'.";
      myrc=3;
   else
      cIHS_ver="${CANDLEHOME}/registry/${cIHSs[0]}";
      cIHS_plat="${cIHSs[0]%.ver}";
      cIHS_plat="${cIHS_plat#iu}";
      if [[ "${cIHS_plat}" = *6 ]]; then
         cIHS_bit="64";
      else
         cIHS_bit="32";
      fi;
      cIHS="${CANDLEHOME}/${cIHS_plat}/iu";
      cIHS_VRMF=$(grep "^VRMF *=" "${cIHS_ver}" | cut -f2 -d'=' | tr -d ' ');
      if [[ "${cIHS_VRMF}" < "08000600" ]]; then
         print -u2 "${pgm}: Current IHS version ${cIHS_VRMF} is too old. Apply ITM fix pack 6.30.02.00 or later first.";
         myrc=3;
      elif [[ ! -r "${cIHS}" ]]; then
         print -u2 "${pgm}: Unable to access ${cIHS}.";
         myrc=3;
      fi;
   fi;                                      # (else($cIHSs...))
   return ${myrc};                          # return code to caller
}                                           # (parse_installation())

function parse_media_image {
   typeset myrc=0;                          # assume success
   #--------------------------------------------------------------------
   # Interpret the install image. Out of this, we get these values:
   #    iWAS      = ${image_dir}/WASIHSRepo_*_v.rr.mm.ff
   #    iWAS_repositories = WASIHSRepo_UNIX_v.rr.mm.ff | WASIHSRepo_LINUX_v.rr.mm.ff
   #    iWAS_VRMF = vvrrmmff derived from ${iWAS} above
   #
   #    iIHS      = ${iWAS}
   #    iIHS_VRMF = ${iWAS_VRMF}
   #    iIHS_repositories = WASIHSRepo_UNIX | WASIHSRepo_LINUX
   #--------------------------------------------------------------------
   image_dir=$(resolve_dir "${image_dir}");
   if (( ${myrc} == 0 )); then
      case ${myos} in
         AIX)   iWASrepoPLAT=UNIX;;
         Linux) iWASrepoPLAT=LINUX;;
      esac;                                    # (case(myos))
      set -A iWASs $(ls -d ${image_dir}/WASIHSRepo_${iWASrepoPLAT}_8.55.[0-9][0-9].[0-9][0-9] 2> /dev/null);
      if (( "${#iWASs[*]}" == 0 )); then
         print -u2 "${pgm}: No WAS version 855 packages found in '${image_dir}' for platform '${myplat}'.";
         myrc=3;
      elif (( "${#iWASs[*]}" > 1 )); then
         print -u2 "${pgm}: Too many WAS version 855 packages found in '${image_dir}' for platform '${myplat}'.";
         myrc=3;
      else
         iWAS="${iWASs[0]}";
         iWAS_repositories=${iWAS};
         iWAS_VRMF=$(basename "${iWAS}" | cut -f3 -d'_' | tr -d '.');
         iWAS_VRMF=$(printf "%08d" "${iWAS_VRMF}");
         iIHS=${iWAS};
         iIHS_repositories=${iWAS_repositories};
         iIHS_VRMF=${iWAS_VRMF};
      fi;                                   # (else($iWASs...))
   fi;                                      # (if($myrc==0))
   return ${myrc};                          # return code to caller
}

function backup_WAS {
   typeset myrc=0;                          # assume success
   typeset rc=0;                            # assume success
   typeset rcIM=0;                          # assume success
   backup_iw_dir="${CANDLEHOME}/${pgm%.*}_iw_backup";
   failed_iw_dir="${CANDLEHOME}/${pgm%.*}_iw_failed";
   #--------------------------------------------------------------------
   # Backup the eWAS 806 files in the <platform>/iw directory in the
   # local environment.
   #--------------------------------------------------------------------

   if [[ "${test_mode}" = "no" ]]; then
      if (( ${update_iw} > 0 )); then
         print "";
         print "${pgm}: Creating ${cWAS_plat}/iw backup in '${backup_iw_dir}'.";
         #--------------------------------------------------------------
         # Get the directories ready.
         #--------------------------------------------------------------
         rm -rf "${backup_iw_dir}";
         rm -rf "${failed_iw_dir}";
         mkdir -p "${backup_iw_dir}";
         rc=${?};
         if (( ${rc} != 0 )); then
            print -u2 "${pgm}: Error creating directory '${backup_iw_dir}' (rc=${rc}).";
            myrc=4;
         fi;                                # (if($rc!=0))
         #--------------------------------------------------------------
         # Backup the cWAS directory and empty it.
         #--------------------------------------------------------------
         if (( ${myrc} == 0 )); then
            find "${cWAS}" -type s -exec rm -f {} \; # rm sockets
            mv "${cWAS}" "${backup_iw_dir}"; # Move the eWAS files
            rc=${?};
            if [[ -d "${cWAS}_IM" ]]; then
               mv "${cWAS}_IM" "${backup_iw_dir}"; # Move the eWAS files
               rcIM=${?};
            fi;
            if (( ${rc} != 0 || ${rcIM} != 0 )); then
               if (( ${rc} != 0 )); then
                  print -u2 "${pgm}: Error moving directory '${cWAS}' (rc=${rc}).";
               fi;
               if (( ${rcIM} != 0 )); then
                  print -u2 "${pgm}: Error moving directory '${cWAS}_IM' (rc=${rcIM}).";
               fi;                          # (else($rc!=0))
               myrc=4;
            else
               backup_iw=1;                 # backup exists now
               print "";
            fi;                             # (else($rc!=0))
            mkdir -p "${cWAS}";             # Create dir mv deleted
            rc=${?};
            mkdir -p "${cWAS}_IM";          # Create dir mv deleted
            rcIM=${?};
            if (( ${rc} != 0 || ${rcIM} != 0 )); then
               if (( ${rc} != 0 )); then
                  print -u2 "${pgm}: Error recreating directory '${cWAS}' (rc=${rc}).";
               fi;                          # (if($rc!=0))
               if (( ${rcIM} != 0 )); then
                  print -u2 "${pgm}: Error recreating directory '${cWAS}_IM' (rc=${rcIM}).";
               fi;                          # (if($rc!=0))
               myrc=4;
            fi;                             # (if($rc!=0))
         fi;                                # (if($myrc==0))
      fi;                                   # (if($update_iw!=0))
   else
      print "";
      print "${pgm}: Skip creating ${cWAS_plat}/iw backup in '${backup_iw_dir}/iw' in test mode.";
   fi;                                      # (else($test_mode=no))
   return ${myrc};                          # return code to caller
}                                           # (backup_WAS())

function backup_IHS {
   typeset myrc=0;                          # assume success
   typeset rc=0;                            # assume success
   typeset rcIM=0;                          # assume success
   backup_iu_dir="${CANDLEHOME}/${pgm%.*}_iu_backup";
   failed_iu_dir="${CANDLEHOME}/${pgm%.*}_iu_failed";
   #--------------------------------------------------------------------
   # Backup the IHS 806 files in the <platform>/iu directory in the
   # local environment.
   #--------------------------------------------------------------------

   if [[ "${test_mode}" = "no" ]]; then
      if (( ${update_iu} > 0 )); then
         print "";
         print "${pgm}: Creating ${cIHS_plat}/iu backup in '${backup_iu_dir}/iu'.";
         #--------------------------------------------------------------
         # Get the directories ready.
         #--------------------------------------------------------------
         rm -rf "${backup_iu_dir}";
         rm -rf "${failed_iu_dir}";
         mkdir "${backup_iu_dir}";
         rc=${?};
         if (( ${rc} != 0 )); then
            print -u2 "${pgm}: Error creating directory '${backup_iu_dir}' (rc=${rc}).";
            myrc=4;
         fi;                                # (if($rc!=0))
         #--------------------------------------------------------------
         # Backup the cIHS directory and empty it.
         #--------------------------------------------------------------
         if (( ${myrc} == 0 )); then
            find "${cIHS}" -type s -exec rm -f {} \; # rm sockets
            mv "${cIHS}" "${backup_iu_dir}"; # Move the IHS files
            rc=${?};
            if [[ -d "${cIHS}_IM" ]]; then
               mv "${cIHS}_IM" "${backup_iu_dir}"; # Move the IHS files
               rcIM=${?};
            fi;
            if (( ${rc} != 0 || ${rcIM} != 0 )); then
               if (( ${rc} != 0 )); then
                  print -u2 "${pgm}: Error copying directory '${cIHS}' (rc=${rc}).";
               fi;                          # (else($rc!=0))
               if (( ${rcIM} != 0 )); then
                  print -u2 "${pgm}: Error copying directory '${cIHS}_IM' (rc=${rc}).";
               fi;                          # (else($rc!=0))
               myrc=4;
            else
               backup_iu=1;                 # backup exists now
               print "";
            fi;                             # (else($rc!=0))
            mkdir -p "${cIHS}";             # Create dir mv deleted
            rc=${?};
            mkdir -p "${cIHS}_IM";          # Create dir mv deleted
            rcIM=${?};
            if (( ${rc} != 0 || ${rcIM} != 0 )); then
               if (( ${rc} != 0 )); then
                  print -u2 "${pgm}: Error recreating directory '${cIHS}' (rc=${rc}).";
               fi;                          # (if($rc!=0))
               if (( ${rcIM} != 0 )); then
                  print -u2 "${pgm}: Error recreating directory '${cIHS}_IM' (rc=${rc}).";
               fi;                          # (if($rc!=0))
               myrc=4;
            fi;                             # (if($rc!=0))
         fi;                                # (if($myrc==0))
      fi;                                   # (if($update_ii!=0))
   else
      print "";
      print "${pgm}: Skip creating ${cIHS_plat}/iu backup in '${backup_iu_dir}/iu' in test mode.";
   fi;                                      # (else($test_mode=no))
   return ${myrc};                          # return code to caller
}                                           # (backup_IHS())

function restore_WAS {
   typeset myrc=0;                          # assume success
   typeset rc=0;                            # assume success
   typeset rcIM=0;                          # assume success
   #--------------------------------------------------------------------
   # Restore the WAS files in the <platform>/iw directory in the
   # local environment.
   #--------------------------------------------------------------------

   print "";
   if ((${backup_iw} == 1)); then           # update ok, del backup
      print "${pgm}: Deleting backup in '${backup_iw_dir}/iw'.";
      rm -rf "${backup_iw_dir}";
      rc=${?};
      if (( ${rc} != 0 )); then
         print -u2 "${pgm}: Error deleting directory '${backup_iw_dir}' (rc=${rc}).";
         myrc=4;
      else
         print -u2 "${pgm}: '${backup_iw_dir}' deleted.";
      fi;
   elif ((${backup_iw} == 2)); then         # update failed, restore
      print "${pgm}: Restoring ${cWAS_plat}/iw from backup in '${backup_iw_dir}/iw'.";
      mkdir "${failed_iw_dir}";             # place to hold failed data
      rc=${?};
      if (( ${rc} != 0 )); then
         print -u2 "${pgm}: Error creating directory '${failed_iw_dir}' (rc=${rc}).";
         myrc=4;
      else
         mv "${cWAS}" "${cWAS}_IM" "${failed_iw_dir}"; # save off broken attempt
         rc=${?};
         if (( ${rc} != 0 )); then
            print -u2 "${pgm}: Error moving directory '${cWAS}' to '${failed_iw_dir}' (rc=${rc}).";
            myrc=4;
         else
            mv "${backup_iw_dir}/iw" "${cWAS}"; # restore original WAS
            rc=${?};
            if [[ -d "${backup_iw_dir}/iw_IM" ]]; then
               mv "${backup_iw_dir}/iw_IM" "${cWAS}_IM"; # restore original WAS
               rcIM=${?};
            fi;
            if (( ${rc} != 0 || ${rcIM} != 0 )) ; then
               if (( ${rc} != 0 )) ; then
                  print -u2 "${pgm}: Error moving directory '${backup_iw_dir}/iw' to '${cWAS}' (rc=${rc}).";
               fi;
               if (( ${rcIM} != 0 )) ; then
                  print -u2 "${pgm}: Error moving directory '${backup_iw_dir}/iw_IM' to '${cWAS}_IM' (rc=${rcIM}).";
               fi;
               myrc=4;
         else
               rmdir "${backup_iw_dir}";
               print "${pgm}: Backup restored to ${cWAS_plat}/iw.\r\n";
               print "${pgm}: Data from failed attempt kept in ${failed_iw_dir}.";
            fi;                             # (else($rc!=0))
         fi;                                # (else($rc!=0))
      fi;                                   # (else($rc!=0))
   fi;                                      # (elif($backup_iw...))
   reportVRMF "iw";
   return ${myrc};                          # return code to caller
}                                           # (restore_WAS())

function restore_IHS {
   typeset myrc=0;                          # assume success
   typeset rc=0;                            # assume success
   typeset rcIM=0;                          # assume success
   #--------------------------------------------------------------------
   # Restore the IHS files in the <platform>/iu directory in the
   # local environment.
   #--------------------------------------------------------------------

   print "";
   if ((${backup_iu} == 1)); then           # update ok, del backup
      print "${pgm}: Deleting backup in '${backup_iu_dir}/iu'.";
      rm -rf "${backup_iu_dir}";
      rc=${?};
      if (( ${rc} != 0 )); then
         print -u2 "${pgm}: Error deleting directory '${backup_iu_dir}' (rc=${rc}).";
         myrc=4;
      else
         print -u2 "${pgm}: '${backup_iu_dir}' deleted.";
      fi;
      #Clean up IHS 806 artifacts if they exist
      typeset IMdir="${CANDLEHOME}/IM";
      rm -rf "${IMdir}";
      rc=${?};
      if (( ${rc} != 0 )); then
         print -u2 "${pgm}: Error deleting directory '${IMdir}' (rc=${rc}).";
      else
         print -u2 "${pgm}: '${IMdir}' deleted.";
      fi;
   elif ((${backup_iu} == 2)); then         # update failed, restore
      print "${pgm}: Restoring ${cIHS_plat}/iu from backup in '${backup_iu_dir}/iu'.";
      mkdir "${failed_iu_dir}";             # place to hold failed data
      rc=${?};
      if (( ${rc} != 0 )); then
         print -u2 "${pgm}: Error creating directory '${failed_iu_dir}' (rc=${rc}).";
         myrc=4;
      else
         mv "${cIHS}" "${cIHS}_IM" "${failed_iu_dir}"; # save off broken attempt
         rc=${?};
         if (( ${rc} != 0 )); then
            print -u2 "${pgm}: Error moving directory '${cIHS}' to '${failed_iu_dir}' (rc=${rc}).";
            myrc=4;
         else
            mv "${backup_iu_dir}/iu" "${cIHS}"; # restore original WAS
            rc=${?};
            if [[ -d "${backup_iu_dir}/iu_IM" ]]; then
               mv "${backup_iu_dir}/iu_IM" "${cIHS}_IM"; # restore original WAS
               rcIM=${?};
            fi;
            if (( ${rc} != 0 || ${rcIM} != 0 )); then
               if (( ${rc} != 0 )); then
                  print -u2 "${pgm}: Error moving directory '${backup_iu_dir}/iu' to '${cIHS}' (rc=${rc}).";
               fi;
               if (( ${rcIM} != 0 )); then
                  print -u2 "${pgm}: Error moving directory '${backup_iu_dir}/iu_im' to '${cIHS}_IM' (rc=${rcIM}).";
               fi;
               myrc=4;
         else
               rmdir "${backup_iu_dir}";
               print "${pgm}: Backup restored to ${cIHS_plat}/iu.\r\n";
               print "${pgm}: Data from failed attempt kept in ${failed_iu_dir}.";
            fi;                             # (else($rc!=0))
         fi;                                # (else($rc!=0))
      fi;                                   # (else($rc!=0))
   fi;                                      # (elif($backup_iu...))
   reportVRMF "iu";
   return ${myrc};                          # return code to caller
}                                           # (restore_IHS())

function install_WAS {
   typeset myrc=0;                          # assume success
   typeset cmd="";
   typeset iimdataloc="${CANDLEHOME}/${cWAS_plat}/iw_IM";
   #--------------------------------------------------------------------
   # Use IIM to install WAS in the local environment.
   #--------------------------------------------------------------------

   #--------------------------------------------------------------------
   # We chose CANDLEHOME/<platform>/iw_IM for the datalocation so;
   # 1) secureMain manages its permissions.
   # 2) uninstall.sh can discard it if the TEPS is removed.
   # IIM requires that the datalocation reside outside of the 
   # installation directory.
   #--------------------------------------------------------------------
   # For the TEPS to work properly, WAS needs IBM's unrestricted
   # policy.jar files. We no longer copy the unrestricted policy.jar
   # from the ITM installation JRE into the WAS/IHS installation since
   # WAS/IHS 855 now contains Java v8 which provided unrestricted
   # policy.jar files.
   #--------------------------------------------------------------------
   # Install WAS 855 from the repos
   #--------------------------------------------------------------------
   if [[ "${test_mode}" = "no" ]]; then
      #-----------------------------------------------------------------
      # Install WAS 855 from the repos
      #-----------------------------------------------------------------
      if (( ${update_iw} > 0 )); then
         print "";
         print "${pgm}: Installing ${cWAS_plat}/iw.";
         if (( ${myrc} == 0 )); then
            cmd="${imcl_cmd}";
            cmd="${cmd} install com.ibm.websphere.BASE.v85";
            cmd="${cmd} -dataLocation '${iimdataloc}/Data'";
            cmd="${cmd} -sharedResourcesDirectory '${iimdataloc}/Shared'";
            cmd="${cmd} -repositories '${iWAS_repositories}'";
            cmd="${cmd} -installationDirectory '${cWAS}' -eclipseLocation '${cWAS}'";
            cmd="${cmd} -acceptLicense -showProgress";
            cmd="${cmd} -preferences com.ibm.cic.common.core.preferences.preserveDownloadedArtifacts=false";
            print "";
            print "${pgm}: Installing WAS packages from image.";
            print "${pgm}: Command: ${cmd}";
            # WAS 855 uplift is a new install of WAS. Do not use old IIM data
            rm -f "${iimdataloc}" 2> /dev/null ;
            eval ${cmd};
            rc=${?};
            # WAS 855 uplift is a new install of WAS. Do not move IIM data
            if (( ${rc} != 0 )); then
               print -u2 "${pgm}: Error installing WAS packages from '${iWAS}' (rc=${rc}).";
               myrc=5;
            else
               print "${pgm}: Installation of WAS package complete.";
            fi;                          # (else($rc!=0))
         fi;                             # (if($myrc==0))
         #-----------------------------------------------------------------
         # List installed WAS 855 packages
         #-----------------------------------------------------------------
         if (( ${myrc} == 0 )); then
            cmd="${imcl_cmd}";
            cmd="${cmd} listInstalledPackages -long";
            cmd="${cmd} -dataLocation '${iimdataloc}/Data'";
            print "";
            print "${pgm}: Listing installed WAS packages.";
            print "${pgm}: Command: ${cmd}";
            WASnames="$(eval ${cmd})";
            rc=${?};
            if (( ${rc} != 0 )); then
               print -u2 "${pgm}: Error listing installed WAS package names (rc=${rc}).";
               myrc=5;
            else
               if [[ -z "${WASnames}" ]]; then
                  print -u2 "${pgm}: Error listing installed WAS package names (rc=${rc}).";
                  myrc=5;
               else
                  print "${pgm}: Installed WAS package names:";
                  print "${WASnames}" | while read line ; do
                     [[ "${line}" == @(*CRIMA1263W*|*symbolic links*|* WARNING *) ]] && continue
                     print "${pgm}:    $(print ${line} | sed 's/[^:]*: //')";
                  done
               fi;                          # (else($WASnames=""))
            fi;                             # (else($rc!=0))
         fi;                                # (if($myrc==0))
         if (( ${myrc} == 0 )); then
            print "${pgm}: ${cWAS_plat}/iw install succeeded.";
         else
            print -u2 "${pgm}: ${cWAS_plat}/iw install failed.";
            backup_iw=2;                    # restore from backup
         fi;
         #-----------------------------------------------------------------
         # Finally, we need to update the ITM ver file to show the
         # new WAS version and its install date.
         #-----------------------------------------------------------------
         if (( ${myrc} == 0 )); then
            update_VerFile ${cWAS_ver} ${iWAS_VRMF};
            rc=${?};
         fi;                                # (if($myrc==0))
      fi;                                   # (if(#update_iw!=0))
   else
      print "";
      print "${pgm}: Skip installing WAS packages from image in test mode.";
   fi;                                      # (else($test_mode==no))
   return ${myrc};                          # return code to caller
}                                           # (install_WAS())

function install_IHS {
   typeset myrc=0;                          # assume success
   typeset cmd="";
   typeset iimdataloc="${CANDLEHOME}/${cIHS_plat}/iu_IM";
   #--------------------------------------------------------------------
   # Use IIM to install IHS and plugin in the local environment.
   #--------------------------------------------------------------------

   #--------------------------------------------------------------------
   # We chose CANDLEHOME/<platform>/iu_IM for the datalocation so;
   # 1) secureMain manages its permissions.
   # 2) uninstall.sh can discard it if the TEPS is removed.
   # IIM requires that the datalocation reside outside of the 
   # installation directory.
   #--------------------------------------------------------------------
   if [[ "${test_mode}" = "no" ]]; then
      #-----------------------------------------------------------------
      # Install IHS 855 from the repos
      # Install IHS plugin 855 from the repos
      #-----------------------------------------------------------------
      if (( ${update_iu} > 0 )); then
         print "";
         print "${pgm}: Installing ${cIHS_plat}/iu.";
         # Install IHS 855 from the repos
         if (( ${myrc} == 0 )); then
            cmd="${imcl_cmd}";
            cmd="${cmd} install com.ibm.websphere.IHS.v85";
            cmd="${cmd} -dataLocation '${iimdataloc}/Data'";
            cmd="${cmd} -sharedResourcesDirectory '${iimdataloc}/Shared'";
            cmd="${cmd} -repositories '${iIHS_repositories}'";
            cmd="${cmd} -installationDirectory '${cIHS}/ihs/HTTPServer' -eclipseLocation '${cIHS}/ihs/HTTPServer'";
            cmd="${cmd} -acceptLicense -showProgress";
            cmd="${cmd} -preferences com.ibm.cic.common.core.preferences.preserveDownloadedArtifacts=false";
            cmd="${cmd} -properties user.ihs.httpPort=80";
            print "";
            print "${pgm}: Installing IHS packages from image.";
            print "${pgm}: Command: ${cmd}";
            # IHS 855 uplift is a new install of IHS. Do not use old IIM data
            eval ${cmd};
            rc=${?};
            # IHS 855 uplift is a new install of IHS. Do not move IIM data
            if (( ${rc} != 0 )); then
               print -u2 "${pgm}: Error installing IHS packages from '${iIHS}' (rc=${rc}).";
               myrc=5;
            else
               print "${pgm}: Installation of IHS package complete.";
            fi;                          # (else($rc!=0))
         fi;                             # (if($myrc==0))
         # Install plugin 855 from the repos
         if (( ${myrc} == 0 )); then
            cmd="${imcl_cmd}";
            cmd="${cmd} install com.ibm.websphere.PLG.v85";
            cmd="${cmd} -dataLocation '${iimdataloc}/Data'";
            cmd="${cmd} -sharedResourcesDirectory '${iimdataloc}/Shared'";
            cmd="${cmd} -repositories '${iIHS_repositories}'";
            cmd="${cmd} -installationDirectory '${cIHS}/ihs/Plugins' -eclipseLocation '${cIHS}/ihs/Plugins'";
            cmd="${cmd} -acceptLicense -showProgress";
            cmd="${cmd} -preferences com.ibm.cic.common.core.preferences.preserveDownloadedArtifacts=false";
            print "";
            print "${pgm}: Installing IHS plugin packages from image.";
            print "${pgm}: Command: ${cmd}";
            eval ${cmd};
            rc=${?};
            if (( ${rc} != 0 )); then
               print -u2 "${pgm}: Error installing IHS plugin packages from '${iIHS}' (rc=${rc}).";
               myrc=5;
            else
               print "${pgm}: Installation of IHS plugin package complete.";
            fi;                          # (else($rc!=0))
         fi;                             # (if($myrc==0))
         #-----------------------------------------------------------------
         # List installed IHS 855 and IHS plugin 855 packages
         #-----------------------------------------------------------------
         if (( ${myrc} == 0 )); then
            cmd="${imcl_cmd}";
            cmd="${cmd} listInstalledPackages -long";
            cmd="${cmd} -dataLocation '${iimdataloc}/Data'";
            print "";
            print "${pgm}: Listing installed IHS and plugin packages.";
            print "${pgm}: Command: ${cmd}";
            IHSnames="$(eval ${cmd})";
            rc=${?};
            if (( ${rc} != 0 )); then
               print -u2 "${pgm}: Error listing installed IHS and plugin package names (rc=${rc}).";
               myrc=5;
            else
               if [[ -z "${IHSnames}" ]]; then
                  print -u2 "${pgm}: Error listing installed IHS and plugin package names (rc=${rc}).";
                  myrc=5;
               else
                  print "${pgm}: Installed IHS and plugin package names:";
                  print "${IHSnames}" | while read line ; do
                     [[ "${line}" == @(*CRIMA1263W*|*symbolic links*|* WARNING *) ]] && continue
                     print "${pgm}:    $(print ${line} | sed 's/[^:]*: //')";
                  done
               fi;                             # (else($pkgname=""))
            fi;                                # (else($rc!=0))
         fi;                                   # (if($myrc==0))
         #-----------------------------------------------------------------
         # Finally, we need to update the ITM ver file to show the
         # new IHS version and its install date.
         #-----------------------------------------------------------------
         if (( ${myrc} == 0 )); then
            update_VerFile ${cIHS_ver} ${iIHS_VRMF};
            rc=${?};
         fi;                                # (if($myrc==0))
         if (( ${myrc} == 0 )); then
            print "${pgm}: ${cIHS_plat}/iu install succeeded.";
         else
            print -u2 "${pgm}: ${cIHS_plat}/iu install failed.";
            backup_iu=2;                    # restore from backup
         fi;
      fi;                                # (if(#update_iw!=0))
   else
      print "";
      print "${pgm}: Skip installing IHS packages from image in test mode.";
      print "${pgm}: Skip installing IHS plugin packages from image in test mode.";
   fi;                                   # (else($test_mode==no))
   return ${myrc};                          # return code to caller
}                                           # (install_IHS())

function configure_WAS_IHS {
   typeset myrc=0;                          # assume success
   typeset cmd;
   typeset JARDIR=${CANDLEHOME}/classes;
   typeset JARFILE="WAS_IHS_855_Config.jar";
   typeset JARCLASS="com.ibm.tivoli.itm.WASIH855.WAS855PostInstall";
   typeset PROPDIR=${CANDLEHOME}/config;
   typeset PROPFILE="WAS_IHS_855_RAS.properties";
   typeset TRACE_LEVEL="DEBUG_MAX";
   #--------------------------------------------------------------------
   # Run the post-install commands required for WAS.
   #--------------------------------------------------------------------
   if [[ "${test_mode}" = "no" ]]; then
      if (( ${update_iw} > -1 && ${update_iu} > -1 )); then
         cmd="${javaCMD}";
         cmd="${cmd} -DGlobalDbgLevel=$TRACE_LEVEL -DInstallRASConfig=${PROPDIR}/${PROPFILE}";
         cmd="${cmd} -cp ${JARDIR}/kjrall.jar:${JARDIR}/jlog.jar:${JARDIR}/icu4j-54_2.jar:${JARDIR}/${JARFILE}";
         cmd="${cmd} ${JARCLASS}";
         cmd="${cmd} -h ${CANDLEHOME} -w ${backup_iw_dir} -i ${backup_iu_dir}";
         print "";
         print "${pgm}: Post configuration of WAS and IHS.";
         print "${pgm}: Command: ${cmd}";
         eval ${cmd};
         rc=${?};
         if (( ${rc} != 0 )); then
            print -u2 "${pgm}: Error configuring WAS (rc=${rc}).";
            myrc=5;
         fi;                          # (else($rc!=0))
         print "";
         print "${pgm}: WAS and IHS configuration completed.";
      else
         print "";
         print "${pgm}: Skipping Post configuration of WAS and IHS because 32 bit upgrade is not supported.";
      fi;                                      # (else($test_mode==no))
   else
      print "";
      print "${pgm}: Skipping Post configuration of WAS and IHS in test mode.";
   fi;                                      # (else($test_mode==no))
   return ${myrc};                          # return code to caller
}                                           # (configure_WAS_IHS())

function main {

   #====================================================================
   # main
   #====================================================================

   #--------------------------------------------------------------------
   # Initialize, parse and validate the parameter list.
   #--------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      parse_parameters "${@}";
      main_rc=${?};
   fi

   #--------------------------------------------------------------------
   # Find the currently installed versions of eWAS and IHS.
   #--------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      parse_installation;
      main_rc=${?};
   fi

   #--------------------------------------------------------------------
   # Make sure we have authority to install.
   #--------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      verify_auth;
      main_rc=${?};
   fi

   #-----------------------------------------------------------------------
   # Find the install image.
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      parse_media_image;
      main_rc=${?};
   fi

   #-----------------------------------------------------------------------
   # Determine whether components needs to be updated.
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      determine_updates;
      main_rc=${?};
   fi

   #-----------------------------------------------------------------------
   # Find the imcl command.
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      find_imcl;
      main_rc=${?};
   fi;                                         # (if($main_rc==0))

   #-----------------------------------------------------------------------
   # Start logging.
   #-----------------------------------------------------------------------
   logging_start;
   main_rc=${?};

   #-----------------------------------------------------------------------
   # Make sure the TEPS isn't running.
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      check_TEPS_running;
      main_rc=${?};
   fi

   #-----------------------------------------------------------------------
   # Stop here if ITM components are running.
   #-----------------------------------------------------------------------
   if (( ${main_rc} != 0 )); then
      logging_stop;
      return;
   fi;                                         # (if($main_rc!=0))

   #-----------------------------------------------------------------------
   # Create a backup of the current iw directory.
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      backup_WAS
      main_rc=${?};
   fi;                                         # (if($main_rc==0))

   #-----------------------------------------------------------------------
   # Install WAS (or pretend to when in test mode).
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      install_WAS
      main_rc=${?};
   fi;                                         # (if($main_rc==0))

   #-----------------------------------------------------------------------
   # Create a backup of the current iu directory.
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      backup_IHS
      main_rc=${?};
   fi;                                         # (if($main_rc==0))

   #-----------------------------------------------------------------------
   # INSTALL IHS and plugin (or pretend to when in test mode).
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      install_IHS;
      main_rc=${?};
   fi;                                         # (if($main_rc==0))

   #-----------------------------------------------------------------------
   # Configure WAS, IHS and plugin
   #-----------------------------------------------------------------------
   if (( ${main_rc} == 0 )); then
      configure_WAS_IHS;
      main_rc=${?};
   fi;                                         # (if($main_rc==0))

   #-----------------------------------------------------------------------
   # Delete or restore iw backup.
   #-----------------------------------------------------------------------
   restore_WAS;

   #-----------------------------------------------------------------------
   # Delete or restore iu backup.
   #-----------------------------------------------------------------------
   restore_IHS;

   #-----------------------------------------------------------------------
   # Stop writing log file and copy to *_failed dir if possible.
   #-----------------------------------------------------------------------
   logging_stop;
}                                           # (main())

#-----------------------------------------------------------------------
# Establish constants/globals.
#-----------------------------------------------------------------------
unalias -a;                                 # avoid aliased commands
pgm=$(basename "${0}");                     # my program name
pgm_dir=$(dirname "${0}");                  # my program directory
myos=$(uname);                              # which operating system?
mymach=$(uname -m);                         # what kind of machine?
myplat="unknown";                           # compute AIX|XLINUX|ZLINUX
main_rc=0;                                  # overall return code
if [[ "${myos}" = "Linux" ]]; then
   if [[ "${mymach}" = "x86_64" ]]; then   myplat="XLINUX";
   elif [[ "${mymach}" = "i"*"86" ]]; then myplat="XLINUX";
   elif [[ "${mymach}" = "s390"* ]]; then  myplat="ZLINUX";
   fi;
elif [[ "${myos}" = "AIX" ]]; then
   myplat="AIX";
fi;                                         # (elif($myos==AIX))

if [[ "${myplat}" = "unknown" ]]; then
   print -u2 "${pgm}: Platform ${myos} ${mymach} is not supported.";
   main_rc=2;
fi;                                         # (if($myplat==unknown))
id_cmd="id";
myeuid=$(${id_cmd} -un);                        # my effective userid
mygrps=$(groups "${myeuid}" | sed 's/.*: *//'); # my groups

backup_iw=0;                                # currently no backup
backup_iu=0;                                # currently no backup

update_iw=0;                                # assume not updating
update_iu=0;                                # assume not updating

test_mode="no";                             # assume live mode

main "${@}";

exit ${main_rc};                            # exit with return code
